<?php
/**
 * Plugin Name: Display Terms Shortcode
 * Plugin URI:  https://github.com/seothemes/display-terms-shortcode/
 * Description: Display a list of terms using the [display-terms] shortcode.
 * Version:     1.0.4
 * Author:      SEO Themes
 * Author URI:  https://seothemes.com/
 * License:     GPL-3.0-or-later
 * License URI: http://www.gnu.org/licenses/gpl-3.0.txt
 * Text Domain: display-terms-shortcode
 * Domain Path: /languages
 */

namespace SeoThemes\DisplayTermsShortcode;

// Prevent direct file access.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

\add_shortcode( 'display-terms', __NAMESPACE__ . '\add_shortcode' );
/**
 * Create the shortcode.
 *
 * @since 1.0.0
 *
 * @param  array $atts Shortcode attributes.
 *
 * @return string
 */
function add_shortcode( $atts ) {
	$atts = shortcode_atts( [
		'taxonomy'               => 'category',
		'orderby'                => 'name',
		'order'                  => 'ASC',
		'hide_empty'             => true,
		'include'                => 'all',
		'exclude'                => 'all',
		'exclude_tree'           => 'all',
		'number'                 => false,
		'offset'                 => '',
		'fields'                 => 'all',
		'name'                   => '',
		'slug'                   => '',
		'hierarchical'           => true,
		'search'                 => '',
		'name__like'             => '',
		'description__like'      => '',
		'pad_counts'             => false,
		'get'                    => '',
		'child_of'               => false,
		'childless'              => false,
		'cache_domain'           => 'core',
		'update_term_meta_cache' => true,
		'meta_query'             => '',
		'meta_key'               => [],
		'meta_value'             => '',
		'show_link'              => true,
		'show_name'              => true,
		'show_description'       => false,
		'show_count'             => false,
		'show_image'             => true,
		'image_size'             => 'full',
		'post_type'              => 'post',
		'post_include'           => [],
		'post_exclude'           => [],
		'post_order'             => 'DESC',
		'post_orderby'           => 'modified',
		'number_to_search'       => 10,
		'parent_element'         => 'ul',
		'child_element'          => 'li',
		'class_prefix'           => 'terms',
	], $atts, 'display-terms' );

	$prefix = esc_attr( $atts['class_prefix'] );
	$terms  = get_terms( $atts );
	$open   = sprintf( '<%s class="%s-list">', $atts['parent_element'], $prefix );
	$output = apply_filters( 'display_terms_shortcode_open', $open, $atts );

	foreach ( $terms as $term ) {
		$inner_open = sprintf( '<%s class="%s-list-item %s">', $atts['child_element'], $atts['class_prefix'], esc_attr( $term->slug ) );
		$output     .= apply_filters( 'display_terms_shortcode_inner_open', $inner_open, $atts, $term );

		if ( $atts['show_link'] ) {
			$link_open = sprintf( '<a href="%s" class="%s-link">', esc_url( get_term_link( $term->term_id ) ), $prefix );
			$output    .= apply_filters( 'display_terms_shortcode_link_open', $link_open, $atts, $term );
		}

		if ( $atts['show_name'] ) {
			$name   = sprintf( '<b class="%s-name">%s</b>', $prefix, esc_html( $term->name ) );
			$output .= apply_filters( 'display_terms_shortcode_name', $name, $atts, $term );
		}

		if ( $atts['show_description'] && ! empty( $term->description ) ) {
			$description = sprintf( '<p class="%s-description">%s</p>', $prefix, wp_kses_post( $term->description ) );
			$output      .= apply_filters( 'display_terms_shortcode_description', $description, $atts, $term );
		}

		if ( $atts['show_count'] ) {
			$count  = sprintf( '<span class="%s-count">%s</span>', $prefix, esc_html( $term->count ) );
			$output .= apply_filters( 'display_terms_shortcode_count', $count, $atts, $term );
		}

		if ( $atts['show_image'] ) {
			$posts = get_posts( apply_filters( 'display_terms_shortcode_latest', [
				'posts_per_page' => $atts['number_to_search'],
				'post_type'      => $atts['post_type'],
				'include'        => $atts['post_include'],
				'exclude'        => $atts['post_exclude'],
				'order'          => $atts['post_order'],
				'orderby'        => $atts['post_orderby'],
				'tax_query'      => [
					[
						'taxonomy' => $term->taxonomy,
						'terms'    => $term->slug,
						'field'    => 'slug',
					],
				],
			] ) );

			foreach ( $posts as $post ) {
				$image_id = apply_filters( 'display_terms_shortcode_image_id', get_post_thumbnail_id( $post->ID ), $term->term_id );

				if ( $image_id ) {
					break;
				}
			}

			if ( isset( $image_id ) && $image_id ) {
				$alt    = get_post_meta( $image_id, '_wp_attachment_image_alt', true );
				$attr   = [
					'alt' => $alt ? $alt : $term->name . ' ' . $term->taxonomy,
				];
				$image  = wp_get_attachment_image( $image_id, $atts['image_size'], false, $attr );
				$output .= apply_filters( 'display_terms_shortcode_image', $image, $atts, $term );
			}
		}

		if ( $atts['show_link'] ) {
			$output .= apply_filters( 'display_terms_shortcode_link_close', '</a>', $atts, $term );
		}

		$inner_close = sprintf( '</%s>', $atts['child_element'] );
		$output      .= apply_filters( 'display_terms_shortcode_inner_close', $inner_close, $atts, $term );
	}

	$close  = sprintf( '</%s>', $atts['parent_element'] );
	$output .= apply_filters( 'display_terms_shortcode_close', $close, $atts, $term );

	return apply_filters( 'display_terms_shortcode_output', $output );
}
